"""Formatting different type of data output into HTML."""
import re

import numpy as np
import pandas as pd


def _pandas_dataframe_to_html_initial_set_up(
    data: pd.DataFrame,
    index: bool = False,
):
    if not index:
        data.reset_index(drop=True, inplace=True)
        plus_number = 1
    else:
        plus_number = 2
    return data, plus_number


def _pandas_dataframe_to_html_percentage(
    data: pd.DataFrame, percentage_col: list = None
):
    for col in percentage_col if percentage_col else []:
        if col in list(data):
            data[col] = f"{(100.0 * data[col]).round(1).astype(str)}%"
    return data


def _pandas_dataframe_to_html_special_numbers(data_html: str):
    for _ in ["nan%", "inf%", "-inf%", "nan", "inf", "-inf"]:
        data_html = re.sub(_, "", data_html)
    return data_html


def _pandas_dataframe_to_html_set_width(
    data_html: str,
    width_col: dict = None,
):
    for key, value in width_col.items() if width_col else {}.items():
        data_html = re.sub(
            f'<th>{key}<th style="width: {value}>">{key}', data_html
        )
    return data_html


def _pandas_dataframe_to_html(
    data: pd.DataFrame,
    numeric_format: str = "{:20.2f}",
    numeric_align: str = "right",
    percentage_col: list = None,
    width_col: dict = None,
    table_style: str = "table",
    table_sub_style: str = "table-striped",
    table_font_size: str = "table-sm",
    index: bool = False,
):
    """Convert pandas dataframe to html output.

    Args:
        data (pd.DataFrame): Data to be formatted.
        numeric_format (str, optional):
            Set up format for numerical data. Defaults to "{:20.0f}".
        numeric_align (str, optional):
            Set default alignment for numerical data. Defaults to "right".
        percentage_col (list, optional):
            List of column names to be percenrage output. Defaults to None.
        width_col (dict, optional):
            Provide key as column name and vale as desired column width.
            Defaults to None.
        table_style (str, optional): HTML table style. Defaults to "table".
        table_sub_style (str, optional):
            HTML table style. Defaults to "table-striped".
        table_font_size (str, optional):
            HTML table style. Defaults to "table-sm".
        index (bool, optional):
            Determine if dropping index. Defaults to False.
    """
    data = data.copy()
    data, plus_number = _pandas_dataframe_to_html_initial_set_up(
        data=data, index=index
    )
    data_type = data.dtypes.tolist()
    align_right_list = [
        f"col{str(idx+plus_number)}-{numeric_align}"
        for idx, val in enumerate(data_type)
        if val != np.object
    ]
    align_right_str = (
        f"<table class='{table_style} {table_font_size} {table_sub_style} "
        f"{' '.join(align_right_list)} '>"
    )
    data = _pandas_dataframe_to_html_percentage(
        data=data, percentage_col=percentage_col
    )
    data.columns = [c.replace("_", " ") for c in data.columns]
    data_html = re.sub(
        r'\<table border="1" class="dataframe table table-bordered">',
        align_right_str,
        data.to_html(
            classes="table table-bordered",
            float_format=numeric_format.format,
            index=index,
        ),
    )
    data_html = _pandas_dataframe_to_html_special_numbers(data_html=data_html)
    data_html = _pandas_dataframe_to_html_set_width(
        data_html=data_html, width_col=width_col
    )
    return data_html



"""Main class to create PDF reports."""
import os
from dataclasses import dataclass

import pandas as pd
import pdfkit
import plotly

from datameetpdf.template.basic_html import _basic_html
from datameetpdf.util.html import _pandas_dataframe_to_html

BASIC_CSS = os.path.join(
    os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
    "template",
    "basic.css",
)


@dataclass
class ReportItem:
    """Class for keeping track of an item in report."""

    item_type: str
    formatted_data: str
    title: str = None
    bold_font: bool = True
    font_size: float = 5
    underline: bool = True

    @property
    def pre_b(self) -> str:
        """Return pre string for bold."""
        return "<b>" if self.bold_font else ""

    @property
    def post_b(self) -> str:
        """Return post string for bold."""
        return "</b>" if self.bold_font else ""

    @property
    def pre_font_size_str(self) -> str:
        """Return string for font size."""
        return f"<h{self.font_size}>"

    @property
    def post_font_size_str(self) -> str:
        """Return string for font size."""
        return f"</h{self.font_size}>"

    @property
    def add_underline(self) -> str:
        """Return a line break."""
        return "<hr>" if self.underline else ""


class ReportCreation:
    """Create PDF report."""

    def __init__(
        self, css_template: str = BASIC_CSS, path: str = None, *args, **kwargs
    ) -> None:
        """Pass basic setting by init."""
        self.css_template = css_template
        self.options = {
            "javascript-delay": 1_000,
            "no-stop-slow-scripts": None,
            "debug-javascript": None,
            "page-size": kwargs.get("page-size", "A4"),
            "dpi": kwargs.get("dpi", 400),
            "margin-top": kwargs.get("margin-top", "0.5in"),
            "margin-right": kwargs.get("margin-right", "0.5in"),
            "margin-bottom": kwargs.get("margin-bottom", "0.5in"),
            "margin-left": kwargs.get("margin-left", "0.5in"),
        }
        self.path = path
        self.report_items = []
        self.body_str = ""

    def add_dataframe(
        self,
        data: pd.DataFrame,
        title: str = None,
        numeric_format: str = "{:20.2f}",
        numeric_align: str = "right",
        percentage_col: list = None,
        width_col: dict = None,
        table_style: str = "table",
        table_sub_style: str = "table-striped",
        table_font_size: str = "table-sm",
        index: bool = False,
        bold_font: bool = True,
        font_size: int = 5,
        underline: bool = True,
    ) -> None:
        """Add dataframe to the report.

        Args:
            data (pd.DataFrame): Data to be formatted.
            numeric_format (str, optional):
                Set up format for numerical data. Defaults to "{:20.0f}".
            numeric_align (str, optional):
                Set default alignment for numerical data. Defaults to "right".
            percentage_col (list, optional):
                List of column names to be percenrage output. Defaults to None.
            width_col (dict, optional):
                Provide key as column name and vale as desired column width.
                Defaults to None.
            table_style (str, optional): HTML table style. Defaults to "table".
            table_sub_style (str, optional):
                HTML table style. Defaults to "table-striped".
            table_font_size (str, optional):
                HTML table style. Defaults to "table-sm".
            index (bool, optional):
                Determine if dropping index. Defaults to False.
            bold_font (bool, optional):
                Determine if title is bold. Defaults to True.
            font_size (int, optional):
                font size to be used. Default to be 5.
        """
        self.report_items += [
            ReportItem(
                item_type="dataframe",
                formatted_data=_pandas_dataframe_to_html(
                    data=data,
                    numeric_format=numeric_format,
                    numeric_align=numeric_align,
                    percentage_col=percentage_col,
                    width_col=width_col,
                    table_style=table_style,
                    table_sub_style=table_sub_style,
                    table_font_size=table_font_size,
                    index=index,
                ),
                title=title,
                bold_font=bold_font,
                font_size=font_size,
                underline=underline,
            )
        ]

    def add_plotly(
        self,
        data: plotly.graph_objs,
        title: str = None,
        bold_font: bool = True,
        font_size: int = 5,
        underline: bool = True,
    ) -> None:
        """Add plotly chart to the report."""
        self.report_items += [
            ReportItem(
                item_type="plotly",
                formatted_data=data,
                title=title,
                bold_font=bold_font,
                font_size=font_size,
                underline=underline,
            )
        ]

    def add_break(self, numbers_of_break: int = 1):
        """Add break line.

        Args:
            numbers_of_break (int, optional): Number of breaks. Defaults to 1.
        """
        self.report_items += [
            ReportItem(
                item_type="break_line",
                formatted_data="<br>" * int(numbers_of_break),
            )
        ]

    def add_next_page_break(self):
        """Add next page break."""
        self.report_items += [
            ReportItem(
                item_type="next_page",
                formatted_data='<div style = "display:block; clear:both; page-break-after:always;"></div>',
            )
        ]

    def generate_pdf(self):
        """Generate PDF."""
        for item in self.report_items:
            if item.title:
                self.body_str += (
                    f"{item.pre_font_size_str}"
                    f"{item.pre_b}"
                    f"{item.title}"
                    f"{item.post_b}"
                    f"{item.post_font_size_str}"
                    f"{item.add_underline}"
                )
            self.body_str += item.formatted_data
        pdfkit.from_string(
            input=_basic_html(body_str=self.body_str),
            output_path=self.path,
            css=self.css_template,
            options=self.options,
        )



"""Basic HTML."""


def _basic_html(body_str: str):
    output = (
        f'<!DOCTYPE html><html lang="en-GB">'
        f'<head>'
        f'<script src="https://cdn.rawgit.com/inexorabletash/polyfill/v0.1.42/polyfill.js"></script>'
        f'<script src="https://cdn.rawgit.com/inexorabletash/polyfill/v0.1.42/typedarray.js"></script>'
        f'<script src="https://cdn.plot.ly/plotly-cartesian-latest.min.js"></script>'
        f'</head>'
        f'  <div class="container">'
        f'     <div class="row">'
        f'          <div class="col-12">'
        f'              {body_str}'
        f'          </div>'
        f'     </div>'
        f'  </div>'
    )
    return output


def _plotly_html(body_str: str):
    output = (
        f'<div id="divPlotly"></div>'
        f'<script>'
        f'  var plotly_data = {body_str};'
        f'  Plotly.react("divPlotly", plotly_data.data, plotly_data.layout);'
        f'</script>'
    )
    return output
